function [CDF_LB,CDF_UB,Eval_point] = CONDITIONAL_BOUND(Response,Treatment,MIV,Treatment_base,MIV_base,MTR_flag,method_flag,Eval_point)

% Purpose: 
% Compute the posterior distribution of the lower and upper bounds
% of E( Ym|MIV = j )
% Ym is the conjectured response variable, 
% MIV is the monotone instrumental variable
% -----------------------------------
% Algorithm: 
% "Bayesian Inference with Monotone Instrumental Variables"
% by Hang Qian (2011)
% -----------------------------------
% Usage:
% Response = discretized (observed) response variable, a column vector
% Treatment = discretized treatment variable, a column vector
% MIV = discretized monotone instrumental variables, a column vector
% Treatment_base = one of the values in Response, i.e., Ym in E(Ym|MIV = j)
% MIV_base = one of the values in MIV, i.e., j in E(Ym|MIV = j)
% MTR_flag: 1 = impose monontone treatment response assumption as well, 0 = not impose
% Method_flag: 1 = analytic distribution, 2 = simulated distribution
% Eval_point = the points to evaluate the posterior c.d.f.
% -----------------------------------
% Returns:
% CDF_LB = posterior c.d.f. of the lower bound of E( Ym|MIV = j )
% CDF_UB = posterior c.d.f. of the upper bound of E( Ym|MIV = j )
% -----------------------------------
% Written by Hang Qian, Iowa State University
% Contact me:  matlabist@gmail.com




if nargin < 3; error('Please specify Treatment, Response and MIV'); end;
if nargin < 4; Treatment_base = max(Treatment);disp('Demo: Auto assign Treatment_base'); end;
if nargin < 5; MIV_base = cell(median(MIV));disp('Demo: Auto assign MIV_base'); end;
if nargin < 6; MTR_flag = 1;end;
if nargin < 7; method_flag = 1;end;
if nargin < 8; Eval_point = nan;end;

% Analyze variables
Response = Response(:);
Treatment = Treatment(:);
MIV = MIV(:);
Response_grid = unique(Response);
Treatment_grid = unique(Treatment);
MIV_grid = unique(MIV);

nResponse = length(Response_grid);
nTreatment = length(Treatment_grid);
nMIV = length(MIV_grid);
ngrid = nResponse * nTreatment * nMIV;
nobs = length(MIV);
if max([nResponse,nTreatment,nMIV]) > nobs * 0.8
    disp('It seems that Treatment, Response or MIV are not fully discretized')
end
    
if ngrid > nobs
    disp('Number of grids exceeds observations. It does not work well.')
end

Treatment_base_ind = find(Treatment_grid == Treatment_base);
MIV_base_ind = find(MIV_grid == MIV_base);
if isempty(Treatment_base_ind) || isempty(MIV_base_ind)
    error('Error in Treatment_base or MIV_base')
end

% Posterior Dirichlet
Prior_para = ones(nResponse,nTreatment,nMIV);
frequency = zeros(nResponse,nTreatment,nMIV);
for k = 1:nResponse
    for t = 1:nTreatment
        for m = 1:nMIV
            frequency(k,t,m) = sum((Response == Response_grid(k)) & (Treatment == Treatment_grid(t)) & (MIV == MIV_grid(m)));
        end
    end
end
Post_para = Prior_para + frequency;

% Compute the coefficients in the linear combination
if MTR_flag == 1
    disp('Identification method: MIV + MTR')
    L_combine_LB = [Response_grid(:,ones(Treatment_base_ind,1)), Response_grid(1)*ones(nResponse,nTreatment-Treatment_base_ind)];
    L_combine_UB = [Response_grid(end)*ones(nResponse,Treatment_base_ind-1), Response_grid(:,ones(nTreatment-Treatment_base_ind+1,1))];
else
    disp('Identification method: MIV alone, no MTR')
    L_combine_LB = Response_grid(1)*ones(nResponse,nTreatment);
    L_combine_LB(:,Treatment_base_ind) = Response_grid;
    L_combine_UB = Response_grid(end)*ones(nResponse,nTreatment);
    L_combine_UB(:,Treatment_base_ind) = Response_grid;    
end


% Compute the posterior distribution of lb and ub of E( Ym|MIV = j )
if isnan(Eval_point)
    npoint = 300;
    Eval_point = linspace(Response_grid(1),Response_grid(end),npoint);
else
    npoint = length(Eval_point);
end
CDF_LB = zeros(npoint,1);
CDF_UB = zeros(npoint,1);
if method_flag == 1
    % Analytic distribution
    for r = 1:npoint
        LB_component = zeros(MIV_base_ind,1);
        for m = 1:MIV_base_ind    
            LB_component(m) = DIR_COMBINE(Post_para(:,:,m),L_combine_LB,Eval_point(r));
        end
        CDF_LB(r) = prod(LB_component);
        
        UB_component = zeros(nMIV,1);
        for m = MIV_base_ind:nMIV
            UB_component(m) = DIR_COMBINE(Post_para(:,:,m),L_combine_UB,Eval_point(r));
        end
        CDF_UB(r) = 1 - prod(1-UB_component(MIV_base_ind:end));
    end
else
    % Simulated distribution
    nsim = 10000;    
    LB_sim = zeros(nsim,1);
    UB_sim = zeros(nsim,1);
    for r = 1:nsim
        joint_prob = zeros(nResponse,nTreatment,nMIV);
        joint_prob(:) = DIRICHLET_RND(Post_para(:),1);
        LB_component = zeros(MIV_base_ind,1);
        for m = 1:MIV_base_ind
            marginal_prob_MIV = sum(sum(joint_prob(:,:,m)));
            conditional_prob = joint_prob(:,:,m) / marginal_prob_MIV;
            LB_component(m) = sum(sum(conditional_prob .* L_combine_LB));
        end
        LB_sim(r) = max(LB_component);
        
        UB_component = zeros(nMIV,1);
        for m = MIV_base_ind:nMIV
            marginal_prob_MIV = sum(sum(joint_prob(:,:,m)));
            conditional_prob = joint_prob(:,:,m) / marginal_prob_MIV;
            UB_component(m) = sum(sum(conditional_prob .* L_combine_UB));
        end 
        UB_sim(r) = min(UB_component(MIV_base_ind:end));
        
    end
    
    for r = 1:npoint    
        CDF_LB(r) = mean(LB_sim < Eval_point(r));
        CDF_UB(r) = mean(UB_sim < Eval_point(r));
    end
end



