function [CDF_LB,CDF_UB,Eval_point] = CONDITIONAL_BOUND_2MIV(Response,Treatment,MIV1,MIV2,Treatment_base,MIV1_base,MIV2_base,MTR_flag,method_flag,Eval_point)

% Purpose: 
% Compute the posterior distribution of the lower and upper bounds
% of E( Ym|MIV1 = j, MIV2 = k )
% Ym is the conjectured response variable, 
% MIV1 is the first monotone instrumental variable
% MIV2 is the second monotone instrumental variable
% -----------------------------------
% Algorithm: 
% "Bayesian Inference with Monotone Instrumental Variables"
% by Hang Qian (2011)
% -----------------------------------
% Usage:
% Response = discretized (observed) response variable, a column vector
% Treatment = discretized treatment variable, a column vector
% MIV1 = discretized monotone instrumental variables, a column vector
% MIV2 = discretized monotone instrumental variables, a column vector
% Treatment_base = one of the values in Response, i.e., Ym in E(Ym|MIV = j)
% MIV1_base = one of the values in MIV1, i.e., j in E( Ym|MIV1 = j, MIV2 = k )
% MIV2_base = one of the values in MIV2, i.e., k in E( Ym|MIV1 = j, MIV2 = k )
% MTR_flag: 1 = impose monontone treatment response assumption as well, 0 = not impose
% Method_flag: 1 = analytic distribution, 2 = simulated distribution
% Eval_point = the points to evaluate the posterior c.d.f.
% -----------------------------------
% Returns:
% CDF_LB = posterior c.d.f. of the lower bound of E( Ym|MIV = j )
% CDF_UB = posterior c.d.f. of the upper bound of E( Ym|MIV = j )
% -----------------------------------
% Written by Hang Qian, Iowa State University
% Contact me:  matlabist@gmail.com


if nargin < 4; error('Please specify Treatment, Response and MIV'); end;
if nargin < 5; Treatment_base = max(Treatment);disp('Demo: Auto assign Treatment_base'); end;
if nargin < 6; MIV1_base = cell(median(MIV1));disp('Demo: Auto assign MIV1_base'); end;
if nargin < 7; MIV2_base = cell(median(MIV2));disp('Demo: Auto assign MIV2_base'); end;
if nargin < 8; MTR_flag = 1;end;
if nargin < 9; method_flag = 1;end;
if nargin < 10; Eval_point = nan;end;

% Analyze variables
Response = Response(:);
Treatment = Treatment(:);
MIV1 = MIV1(:);
MIV2 = MIV2(:);
Response_grid = unique(Response);
Treatment_grid = unique(Treatment);
MIV1_grid = unique(MIV1);
MIV2_grid = unique(MIV2);

nResponse = length(Response_grid);
nTreatment = length(Treatment_grid);
nMIV1 = length(MIV1_grid);
nMIV2 = length(MIV2_grid);
ngrid = nResponse * nTreatment * nMIV1 * nMIV2;
nobs = length(MIV1);
if max([nResponse,nTreatment,nMIV1,nMIV2]) > nobs * 0.8
    disp('It seems that Treatment, Response or MIV are not fully discretized')
end
    
if ngrid > nobs
    disp('Number of grids exceeds observations. It does not work well.')
end

Treatment_base_ind = find(Treatment_grid == Treatment_base);
MIV1_base_ind = find(MIV1_grid == MIV1_base);
MIV2_base_ind = find(MIV2_grid == MIV2_base);
if isempty(Treatment_base_ind) || isempty(MIV1_base_ind) || isempty(MIV2_base_ind)
    error('Error in Treatment_base or MIV_base')
end

% Posterior Dirichlet
Prior_para = ones(nResponse,nTreatment,nMIV1,nMIV2);
frequency = zeros(nResponse,nTreatment,nMIV1,nMIV2);
for k = 1:nResponse
    for t = 1:nTreatment
        for m = 1:nMIV1
            for n = 1:nMIV2
                frequency(k,t,m,n) = sum((Response == Response_grid(k)) & (Treatment == Treatment_grid(t)) & (MIV1 == MIV1_grid(m)) & (MIV2 == MIV2_grid(n)));
            end
        end
    end
end
Post_para = Prior_para + frequency;

% Compute the coefficients in the linear combination
if MTR_flag == 1
    disp('Identification method: MIV + MTR')
    L_combine_LB = [Response_grid(:,ones(Treatment_base_ind,1)), Response_grid(1)*ones(nResponse,nTreatment-Treatment_base_ind)];
    L_combine_UB = [Response_grid(end)*ones(nResponse,Treatment_base_ind-1), Response_grid(:,ones(nTreatment-Treatment_base_ind+1,1))];
else
    disp('Identification method: MIV alone, no MTR')
    L_combine_LB = Response_grid(1)*ones(nResponse,nTreatment);
    L_combine_LB(:,Treatment_base_ind) = Response_grid;
    L_combine_UB = Response_grid(end)*ones(nResponse,nTreatment);
    L_combine_UB(:,Treatment_base_ind) = Response_grid;    
end


% Compute the posterior distribution of lb and ub of E(Ym|MIV1 = j, MIV2 = k)
if isnan(Eval_point)
    npoint = 100;
    Eval_point = linspace(Response_grid(1),Response_grid(end),npoint);
else
    npoint = length(Eval_point);
end
CDF_LB = zeros(npoint,1);
CDF_UB = zeros(npoint,1);
if method_flag == 1
    % Analytic distribution
    for r = 1:npoint
        LB_component = zeros(MIV1_base_ind,MIV2_base_ind);
        for m = 1:MIV1_base_ind
            for n = 1:MIV2_base_ind
                LB_component(m,n) = DIR_COMBINE(Post_para(:,:,m,n),L_combine_LB,Eval_point(r));
            end
        end
        CDF_LB(r) = prod(LB_component(:));
        
        UB_component = zeros(nMIV1,nMIV2);
        for m = MIV1_base_ind:nMIV1
            for n = MIV2_base_ind:nMIV2
                UB_component(m,n) = DIR_COMBINE(Post_para(:,:,m,n),L_combine_UB,Eval_point(r));
            end
        end
        CDF_UB(r) = 1 - prod(prod(1-UB_component(MIV1_base_ind:end,MIV2_base_ind:end)));
    end
else
    % Simulated distribution
    nsim = 10000;    
    LB_sim = zeros(nsim,1);
    UB_sim = zeros(nsim,1);
    for r = 1:nsim
        joint_prob = zeros(nResponse,nTreatment,nMIV1,nMIV2);
        joint_prob(:) = DIRICHLET_RND(Post_para(:),1);
        LB_component = zeros(MIV1_base_ind,MIV2_base_ind);
        for m = 1:MIV1_base_ind
            for n = 1:MIV2_base_ind
                marginal_prob_MIV = sum(sum(joint_prob(:,:,m,n)));
                if marginal_prob_MIV > 0
                    conditional_prob = joint_prob(:,:,m,n) / marginal_prob_MIV;
                else
                    conditional_prob = 0;
                end                
                LB_component(m,n) = sum(sum(conditional_prob .* L_combine_LB));
            end
        end
        LB_sim(r) = max(LB_component(:));
        
        UB_component = zeros(nMIV1,nMIV2);
        for m = MIV1_base_ind:nMIV1
            for n = MIV2_base_ind:nMIV2
                marginal_prob_MIV = sum(sum(joint_prob(:,:,m,n)));
                if marginal_prob_MIV > 0
                    conditional_prob = joint_prob(:,:,m,n) / marginal_prob_MIV;
                else
                    conditional_prob = 0;
                end                
                UB_component(m,n) = sum(sum(conditional_prob .* L_combine_UB));
            end
        end 
        UB_sim(r) = min(min(UB_component(MIV1_base_ind:end,MIV2_base_ind:end)));
        
    end
    
    for r = 1:npoint    
        CDF_LB(r) = mean(LB_sim < Eval_point(r));
        CDF_UB(r) = mean(UB_sim < Eval_point(r));
    end
end



