function [LB1_sim,UB1_sim,LB2_sim,UB2_sim,LB_diff_sim,UB_diff_sim] = MARGINAL_BOUND_2MIV(Response,Treatment,MIV1,MIV2,Treatment_base1,Treatment_base2,MTR_flag,nsim)

% Purpose: 
% Compute the posterior distribution of the lower and upper bounds
% of E(Ym),E(Yn),E(Ym)-E(Yn)
% Ym, Yn are the conjectured response variables, 
% -----------------------------------
% Algorithm: 
% "Bayesian Inference with Monotone Instrumental Variables"
% by Hang Qian (2011)
% -----------------------------------
% Usage:
% Response = discretized (observed) response variable, a column vector
% Treatment = discretized treatment variable, a column vector
% MIV1 = discretized monotone instrumental variables, a column vector
% MIV2 = discretized monotone instrumental variables, a column vector
% Treatment_base1 = one of the values in Response, i.e., Ym
% Treatment_base2 = one of the values in Response, i.e., Yn
% MTR_flag: 1 = impose monontone treatment response assumption as well, 0 = not impose
% nsim = number of draws in the simulation
% -----------------------------------
% Returns:
% LB1_sim = posterior draws of the lower bound of E(Ym)
% UB1_sim = posterior draws of the upper bound of E(Ym)
% LB2_sim = posterior draws of the lower bound of E(Yn)
% UB2_sim = posterior draws of the upper bound of E(Yn)
% LB_diff_sim = posterior draws of the lower bound of E(Ym)-E(Yn)
% UB_diff_sim = posterior draws of the upper bound of E(Ym)-E(Yn)
% -----------------------------------
% Written by Hang Qian, Iowa State University
% Contact me:  matlabist@gmail.com




if nargin < 4; error('Please specify Treatment, Response and MIV'); end;
if nargin < 5; Treatment_base1 = max(Treatment);disp('Demo: Auto assign Treatment_base1'); end;
if nargin < 6; Treatment_base2 = min(Treatment);disp('Demo: Auto assign Treatment_base2'); end;
if nargin < 7; MTR_flag = 1;end;
if nargin < 8; nsim = 10000;end;

% Analyze variables
Response = Response(:);
Treatment = Treatment(:);
MIV1 = MIV1(:);
MIV2 = MIV2(:);
Response_grid = unique(Response);
Treatment_grid = unique(Treatment);
MIV1_grid = unique(MIV1);
MIV2_grid = unique(MIV2);

nResponse = length(Response_grid);
nTreatment = length(Treatment_grid);
nMIV1 = length(MIV1_grid);
nMIV2 = length(MIV2_grid);
ngrid = nResponse * nTreatment * nMIV1 * nMIV2;
nobs = length(MIV1);
if max([nResponse,nTreatment,nMIV1,nMIV2]) > nobs * 0.8
    disp('It seems that Treatment, Response or MIV are not fully discretized')
end
    
if ngrid > nobs
    disp('Number of grids exceeds observations. It does not work well.')
end

Treatment_base_ind1 = find(Treatment_grid == Treatment_base1);
Treatment_base_ind2 = find(Treatment_grid == Treatment_base2);
if isempty(Treatment_base_ind1) || isempty(Treatment_base_ind2)
    error('Error in Treatment_base')
end

% Posterior Dirichlet
Prior_para = ones(nResponse,nTreatment,nMIV1,nMIV2);
frequency = zeros(nResponse,nTreatment,nMIV1,nMIV2);
for k = 1:nResponse
    for t = 1:nTreatment
        for m = 1:nMIV1
            for n = 1:nMIV2
                frequency(k,t,m,n) = sum((Response == Response_grid(k)) & (Treatment == Treatment_grid(t)) & (MIV1 == MIV1_grid(m)) & (MIV2 == MIV2_grid(n)));
            end
        end
    end
end
Post_para = Prior_para + frequency;

% Compute the coefficients in the linear combination
if MTR_flag == 1
    disp('Identification method: MIV + MTR')
    Combine_LB1 = [Response_grid(:,ones(Treatment_base_ind1,1)), Response_grid(1)*ones(nResponse,nTreatment-Treatment_base_ind1)];
    Combine_UB1 = [Response_grid(end)*ones(nResponse,Treatment_base_ind1-1), Response_grid(:,ones(nTreatment-Treatment_base_ind1+1,1))];
    
    Combine_LB2 = [Response_grid(:,ones(Treatment_base_ind2,1)), Response_grid(1)*ones(nResponse,nTreatment-Treatment_base_ind2)];
    Combine_UB2 = [Response_grid(end)*ones(nResponse,Treatment_base_ind2-1), Response_grid(:,ones(nTreatment-Treatment_base_ind2+1,1))];
else
    disp('Identification method: MIV alone, no MTR')
    Combine_LB1 = Response_grid(1)*ones(nResponse,nTreatment);
    Combine_LB1(:,Treatment_base_ind1) = Response_grid;
    Combine_UB1 = Response_grid(end)*ones(nResponse,nTreatment);
    Combine_UB1(:,Treatment_base_ind1) = Response_grid;
    
    Combine_LB2 = Response_grid(1)*ones(nResponse,nTreatment);
    Combine_LB2(:,Treatment_base_ind2) = Response_grid;
    Combine_UB2 = Response_grid(end)*ones(nResponse,nTreatment);
    Combine_UB2(:,Treatment_base_ind2) = Response_grid;
    
end

% Simulated distribution
LB1_sim = zeros(nsim,1);
UB1_sim = zeros(nsim,1);
LB2_sim = zeros(nsim,1);
UB2_sim = zeros(nsim,1);
LB_diff_sim = zeros(nsim,1);
UB_diff_sim = zeros(nsim,1);
for r = 1:nsim
    
    % Generate Dirichlet samples, compute marginal and conditional prob.
    joint_prob = zeros(nResponse,nTreatment,nMIV1,nMIV2);
    joint_prob(:) = DIRICHLET_RND(Post_para(:),1);
    marginal_prob_MIV = zeros(nMIV1,nMIV2);
    conditional_prob = zeros(nResponse,nTreatment,nMIV1,nMIV2);
    for m = 1:nMIV1
        for n = 1:nMIV2
            marginal_prob_MIV(m,n) = sum(sum(joint_prob(:,:,m,n)));
            conditional_prob(:,:,m,n) = joint_prob(:,:,m,n) / marginal_prob_MIV(m,n);
        end
    end    
    
    % Take posterior draws of the lower / upper bound of E(Ym)      
    LB_component = zeros(nMIV1,nMIV2);
    UB_component = zeros(nMIV1,nMIV2);
    for m = 1:nMIV1
        for n = 1:nMIV2
            LB_component(m,n) = sum(sum(conditional_prob(:,:,m,n) .* Combine_LB1));
            UB_component(m,n) = sum(sum(conditional_prob(:,:,m,n) .* Combine_UB1));  
        end
    end
    
    LB_conditional = zeros(nMIV1,nMIV2);
    UB_conditional = zeros(nMIV1,nMIV2);
    for m = 1:nMIV1
        for n = 1:nMIV2
            LB_conditional(m,n) = max(max(LB_component(1:m,1:n))); 
            UB_conditional(m,n) = min(min(UB_component(m:end,n:end)));
        end
    end
    LB1_sim(r) = sum(sum(LB_conditional .* marginal_prob_MIV));
    UB1_sim(r) = sum(sum(UB_conditional .* marginal_prob_MIV));
    
%     fprintf('Prob(4,4) = %6.2f, LB = %6.2f \n',marginal_prob_MIV(4,4),LB_conditional(4,4));
%     fprintf('Prob(3,3) = %6.2f, LB = %6.2f \n',marginal_prob_MIV(3,3),LB_conditional(3,3));
%     fprintf('Prob(2,2) = %6.2f, LB = %6.2f \n',marginal_prob_MIV(2,2),LB_conditional(2,2));
%     fprintf('LB_whole = %6.2f \n',LB1_sim(r));
%     disp(' ')
    
    % Take posterior draws of the lower / upper bound of E(Yn)   
    for m = 1:nMIV1
        for n = 1:nMIV2
            LB_component(m,n) = sum(sum(conditional_prob(:,:,m,n) .* Combine_LB2));
            UB_component(m,n) = sum(sum(conditional_prob(:,:,m,n) .* Combine_UB2)); 
        end
    end
    
    for m = 1:nMIV1
        for n = 1:nMIV2
            LB_conditional(m,n) = max(max(LB_component(1:m,1:n))); 
            UB_conditional(m,n) = min(min(UB_component(m:end,n:end)));
        end
    end
    LB2_sim(r) = sum(sum(LB_conditional .* marginal_prob_MIV));
    UB2_sim(r) = sum(sum(UB_conditional .* marginal_prob_MIV));
    
    % Take posterior draws of the lower / upper bound of E(Ym) - E(Yn)
    LB_diff_sim(r) = LB1_sim(r) - UB2_sim(r);
    UB_diff_sim(r) = UB1_sim(r) - LB2_sim(r);
        
end
  
