clear all
clc

% This MATLAB scripts implement the application in
% "Bayesian Inference with Monotone Instrumental Variables" by Hang Qian (2011)
% This application quantifies the effects of taking extra classes
% on high school students' test scores. 

myseed = 20110322;
try
    RandStream.setDefaultStream(RandStream('mt19937ar','seed',myseed));
catch
    randn('seed',myseed);
end

[data,header] = xlsread('ASVAB.xlsx');
header = header';
extra_class = data(:,3);
ASVAB = data(:,16);
mother_edu = data(:,17);
father_edu = data(:,18);

mother_edu(mother_edu == 8) = 7;
father_edu(father_edu == 8) = 7;
mask = (ASVAB >= 0) & (extra_class >= 0) & (mother_edu >= 0) & (father_edu >= 0);

%--------------------------------
% Designate treatment variable, response variable and MIV
Response = round(ASVAB(mask) / 10000) *10;
Treatment = extra_class(mask);
MIV1 = mother_edu(mask);
MIV2 = father_edu(mask);

% Descriptive statistics
Response_grid = unique(Response)';
Treatment_grid = unique(Treatment)';
MIV1_grid = unique(MIV1)';
MIV2_grid = unique(MIV2)';

nResponse = length(Response_grid);
nTreatment = length(Treatment_grid);
nMIV1 = length(MIV1_grid);
nMIV2 = length(MIV2_grid);
nobs = length(Response);

Response_stat = round( mean(Response(:,ones(nResponse,1)) == Response_grid(ones(nobs,1),:)) * 100);
Treatment_stat = round( mean(Treatment(:,ones(nTreatment,1)) == Treatment_grid(ones(nobs,1),:)) * 100);
MIV1_stat = round( mean(MIV1(:,ones(nMIV1,1)) == MIV1_grid(ones(nobs,1),:)) * 100);
MIV2_stat = round( mean(MIV2(:,ones(nMIV2,1)) == MIV2_grid(ones(nobs,1),:)) * 100);

disp('------------------')
disp('Summary Statistics')
disp('------------------')
disp(['Sample size: ',num2str(nobs)])
disp('Extra class taking (Treatment variable) ')
disp(['Values ',num2str(Treatment_grid)])
disp(['Percent ',num2str(Treatment_stat)])
disp('ASVAB score (Response variable) ')
disp(['Values ',num2str(Response_grid)])
disp(['Percent ',num2str(Response_stat)])
disp('Mother highest degree (MIV 1) ')
disp('Values None GED High Junior Bachelor Master PhD')
disp(['Percent ',num2str(MIV1_stat)])
disp('Father highest degree (MIV 2) ')
disp('Values None GED High Junior Bachelor Master PhD')
disp(['Percent ',num2str(MIV2_stat)])

stat_mat = NaN(12,max([nResponse,nTreatment,nMIV1,nMIV2]));
stat_mat(2,1:nTreatment) = Treatment_grid;
stat_mat(3,1:nTreatment) = Treatment_stat;
stat_mat(5,1:nResponse) = Response_grid;
stat_mat(6,1:nResponse) = Response_stat;
stat_mat(8,1:nMIV1) = MIV1_grid;
stat_mat(9,1:nMIV1) = MIV1_stat;
stat_mat(11,1:nMIV2) = MIV2_grid;
stat_mat(12,1:nMIV2) = MIV2_stat;
xlswrite('Summary Statistics.xls',stat_mat,'Sheet1','B1');
xlswrite('Summary Statistics.xls',{'Extra class taking (Treatment variable)'},'Sheet1','A1');
xlswrite('Summary Statistics.xls',{'ASVAB score (Response variable)'},'Sheet1','A4');
xlswrite('Summary Statistics.xls',{'Mother highest degree (MIV 1)'},'Sheet1','A7');
xlswrite('Summary Statistics.xls',{'Father highest degree (MIV 2)'},'Sheet1','A10');


% Conditional on both parents have college degrees,
% analytic lower and upper bounds for the expectation of the
% conjectured ASVAB score (Y1, Y0)
MTR_flag = 1;
method_flag = 1;
npoint = 500;
Eval_point = linspace(0,100,npoint)';
MIV1_base = 5;
MIV2_base = 5;
Treatment_base = 1;
[CDF_LB11,CDF_UB11] = CONDITIONAL_BOUND_2MIV(Response,Treatment,MIV1,MIV2,Treatment_base,MIV1_base,MIV2_base,MTR_flag,method_flag,Eval_point);
Treatment_base = 0;
[CDF_LB12,CDF_UB12] = CONDITIONAL_BOUND_2MIV(Response,Treatment,MIV1,MIV2,Treatment_base,MIV1_base,MIV2_base,MTR_flag,method_flag,Eval_point);

% Conditional on both parents are high shool graduates,
% analytic lower and upper bounds for the expectation of the
% conjectured ASVAB score (Y1, Y0)
MIV1_base = 3;
MIV2_base = 3;
Treatment_base = 1;
[CDF_LB21,CDF_UB21] = CONDITIONAL_BOUND_2MIV(Response,Treatment,MIV1,MIV2,Treatment_base,MIV1_base,MIV2_base,MTR_flag,method_flag,Eval_point);
Treatment_base = 0;
[CDF_LB22,CDF_UB22] = CONDITIONAL_BOUND_2MIV(Response,Treatment,MIV1,MIV2,Treatment_base,MIV1_base,MIV2_base,MTR_flag,method_flag,Eval_point);


Prob_LB11 = [0;diff(CDF_LB11)];
Prob_UB11 = [0;diff(CDF_UB11)];
Prob_LB12 = [0;diff(CDF_LB12)];
Prob_UB12 = [0;diff(CDF_UB12)];
Prob_LB21 = [0;diff(CDF_LB21)];
Prob_UB21 = [0;diff(CDF_UB21)];
Prob_LB22 = [0;diff(CDF_LB22)];
Prob_UB22 = [0;diff(CDF_UB22)];

PDF_LB11 = Prob_LB11 / (100 / npoint);
PDF_UB11 = Prob_UB11 / (100 / npoint);
PDF_LB12 = Prob_LB12 / (100 / npoint);
PDF_UB12 = Prob_UB12 / (100 / npoint);
PDF_LB21 = Prob_LB21 / (100 / npoint);
PDF_UB21 = Prob_UB21 / (100 / npoint);
PDF_LB22 = Prob_LB22 / (100 / npoint);
PDF_UB22 = Prob_UB22 / (100 / npoint);

% Summary statistics of the posterior distribution
Mean_LB11 = Prob_LB11' * Eval_point;
Mean_UB11 = Prob_UB11' * Eval_point;
Mean_LB12 = Prob_LB12' * Eval_point;
Mean_UB12 = Prob_UB12' * Eval_point;
Mean_LB21 = Prob_LB21' * Eval_point;
Mean_UB21 = Prob_UB21' * Eval_point;
Mean_LB22 = Prob_LB22' * Eval_point;
Mean_UB22 = Prob_UB22' * Eval_point;

Std_LB11 = sqrt(Prob_LB11' * (Eval_point - Mean_LB11).^2);
Std_UB11 = sqrt(Prob_UB11' * (Eval_point - Mean_UB11).^2);
Std_LB12 = sqrt(Prob_LB12' * (Eval_point - Mean_LB12).^2);
Std_UB12 = sqrt(Prob_UB12' * (Eval_point - Mean_UB12).^2);
Std_LB21 = sqrt(Prob_LB21' * (Eval_point - Mean_LB21).^2);
Std_UB21 = sqrt(Prob_UB21' * (Eval_point - Mean_UB21).^2);
Std_LB22 = sqrt(Prob_LB22' * (Eval_point - Mean_LB22).^2);
Std_UB22 = sqrt(Prob_UB22' * (Eval_point - Mean_UB22).^2);

Median_LB11 = Eval_point( find(CDF_LB11 > 0.5,1) );
Median_UB11 = Eval_point( find(CDF_UB11 > 0.5,1) );
Median_LB12 = Eval_point( find(CDF_LB12 > 0.5,1) );
Median_UB12 = Eval_point( find(CDF_UB12 > 0.5,1) );
Median_LB21 = Eval_point( find(CDF_LB21 > 0.5,1) );
Median_UB21 = Eval_point( find(CDF_UB21 > 0.5,1) );
Median_LB22 = Eval_point( find(CDF_LB22 > 0.5,1) );
Median_UB22 = Eval_point( find(CDF_UB22 > 0.5,1) );

[temp,ind] = max(PDF_LB11); Mode_LB11 = Eval_point(ind);
[temp,ind] = max(PDF_UB11); Mode_UB11 = Eval_point(ind);
[temp,ind] = max(PDF_LB12); Mode_LB12 = Eval_point(ind);
[temp,ind] = max(PDF_UB12); Mode_UB12 = Eval_point(ind);
[temp,ind] = max(PDF_LB21); Mode_LB21 = Eval_point(ind);
[temp,ind] = max(PDF_UB21); Mode_UB21 = Eval_point(ind);
[temp,ind] = max(PDF_LB22); Mode_LB22 = Eval_point(ind);
[temp,ind] = max(PDF_UB22); Mode_UB22 = Eval_point(ind);

alpha = 0.05;
HPD_LB11 = HPD_CDF(Eval_point,CDF_LB11,alpha);
HPD_UB11 = HPD_CDF(Eval_point,CDF_UB11,alpha);
HPD_LB12 = HPD_CDF(Eval_point,CDF_LB12,alpha);
HPD_UB12 = HPD_CDF(Eval_point,CDF_UB12,alpha);
HPD_LB21 = HPD_CDF(Eval_point,CDF_LB21,alpha);
HPD_UB21 = HPD_CDF(Eval_point,CDF_UB21,alpha);
HPD_LB22 = HPD_CDF(Eval_point,CDF_LB22,alpha);
HPD_UB22 = HPD_CDF(Eval_point,CDF_UB22,alpha);

disp(' ')
disp('------------------')
disp('Bounds of conditional mean')
disp('------------------')
fprintf('\t\t\t\t\t  Mean \t Std \t Median \t Mode \t\t HPD\n');
disp('Distribution of lower bounds')
fprintf('E(Y1|Z = Bachelor) \t %5.2f \t %5.2f \t %5.2f \t %5.2f \t [%5.2f , %5.2f] \n',Mean_LB11,Std_LB11,Median_LB11,Mode_LB11,HPD_LB11(1),HPD_LB11(2));
fprintf('E(Y0|Z = Bachelor) \t %5.2f \t %5.2f \t %5.2f \t %5.2f \t [%5.2f , %5.2f] \n',Mean_LB12,Std_LB12,Median_LB12,Mode_LB12,HPD_LB12(1),HPD_LB12(2));
fprintf('E(Y1|Z = High sch) \t %5.2f \t %5.2f \t %5.2f \t %5.2f \t [%5.2f , %5.2f] \n',Mean_LB21,Std_LB21,Median_LB21,Mode_LB21,HPD_LB21(1),HPD_LB21(2));
fprintf('E(Y0|Z = High sch) \t %5.2f \t %5.2f \t %5.2f \t %5.2f \t [%5.2f , %5.2f] \n',Mean_LB22,Std_LB22,Median_LB22,Mode_LB22,HPD_LB22(1),HPD_LB22(2));
disp(' ')
disp('Distribution of upper bounds')
fprintf('E(Y1|Z = Bachelor) \t %5.2f \t %5.2f \t %5.2f \t %5.2f \t [%5.2f , %5.2f] \n',Mean_UB11,Std_UB11,Median_UB11,Mode_UB11,HPD_UB11(1),HPD_UB11(2));
fprintf('E(Y0|Z = Bachelor) \t %5.2f \t %5.2f \t %5.2f \t %5.2f \t [%5.2f , %5.2f] \n',Mean_UB12,Std_UB12,Median_UB12,Mode_UB12,HPD_UB12(1),HPD_UB12(2));
fprintf('E(Y1|Z = High sch) \t %5.2f \t %5.2f \t %5.2f \t %5.2f \t [%5.2f , %5.2f] \n',Mean_UB21,Std_UB21,Median_UB21,Mode_UB21,HPD_UB21(1),HPD_UB21(2));
fprintf('E(Y0|Z = High sch) \t %5.2f \t %5.2f \t %5.2f \t %5.2f \t [%5.2f , %5.2f] \n',Mean_UB22,Std_UB22,Median_UB22,Mode_UB22,HPD_UB22(1),HPD_UB22(2));

Conditional_mat = NaN(9,4);
Conditional_mat(1:4,:) = [Mean_LB11,Std_LB11,Median_LB11,Mode_LB11; Mean_LB12,Std_LB12,Median_LB12,Mode_LB12; Mean_LB21,Std_LB21,Median_LB21,Mode_LB21; Mean_LB22,Std_LB22,Median_LB22,Mode_LB22];
Conditional_mat(6:9,:) = [Mean_UB11,Std_UB11,Median_UB11,Mode_UB11; Mean_UB12,Std_UB12,Median_UB12,Mode_UB12; Mean_UB21,Std_UB21,Median_UB21,Mode_UB21; Mean_UB22,Std_UB22,Median_UB22,Mode_UB22];
HPD_results = cell(9,1);
HPD_results{1} = [ '['  num2str(HPD_LB11(1),'%5.2f')  ' , '  num2str(HPD_LB11(2),'%5.2f')  ']' ];
HPD_results{2} = [ '['  num2str(HPD_LB12(1),'%5.2f')  ' , '  num2str(HPD_LB12(2),'%5.2f')  ']' ];
HPD_results{3} = [ '['  num2str(HPD_LB21(1),'%5.2f')  ' , '  num2str(HPD_LB21(2),'%5.2f')  ']' ];
HPD_results{4} = [ '['  num2str(HPD_LB22(1),'%5.2f')  ' , '  num2str(HPD_LB22(2),'%5.2f')  ']' ];
HPD_results{6} = [ '['  num2str(HPD_UB11(1),'%5.2f')  ' , '  num2str(HPD_UB11(2),'%5.2f')  ']' ];
HPD_results{7} = [ '['  num2str(HPD_UB12(1),'%5.2f')  ' , '  num2str(HPD_UB12(2),'%5.2f')  ']' ];
HPD_results{8} = [ '['  num2str(HPD_UB21(1),'%5.2f')  ' , '  num2str(HPD_UB21(2),'%5.2f')  ']' ];
HPD_results{9} = [ '['  num2str(HPD_UB22(1),'%5.2f')  ' , '  num2str(HPD_UB22(2),'%5.2f')  ']' ];

xlswrite('Conditional distribution.xls',Conditional_mat,'Sheet1','B3');
xlswrite('Conditional distribution.xls',HPD_results,'Sheet1','F3');
xlswrite('Conditional distribution.xls',{'Mean','Std','Median','Mode','HPD'},'Sheet1','B1');
xlswrite('Conditional distribution.xls',{'Distribution of lower bounds'},'Sheet1','A2');
xlswrite('Conditional distribution.xls',{'Distribution of lower bounds';'E(Y1|Z = Bachelor)';'E(Y0|Z = Bachelor)';'E(Y1|Z = High school)';'E(Y0|Z = High school)';...
    'Distribution of upper bounds';'E(Y1|Z = Bachelor)';'E(Y0|Z = Bachelor)';'E(Y1|Z = High school)';'E(Y0|Z = High school)'},'Sheet1','A2');


figure(1)
subplot(2,2,1);
plot(Eval_point,PDF_LB11,'-b',Eval_point,PDF_UB11,'--r'),title('Bounds of E(Y_1 | Z = Bachelor)'),xlim([30,80]),ylim([0,0.3]);
legend('Lower bound', 'Upper bound','location','Northwest');
subplot(2,2,2);
plot(Eval_point,PDF_LB12,'-b',Eval_point,PDF_UB12,'--r'),title('Bounds of E(Y_0 | Z = Bachelor)'),xlim([30,80]),ylim([0,0.3]);
subplot(2,2,3);
plot(Eval_point,PDF_LB21,'-b',Eval_point,PDF_UB21,'--r'),title('Bounds of E(Y_1 | Z = High)'),xlim([30,80]),ylim([0,0.3]);
subplot(2,2,4);
plot(Eval_point,PDF_LB22,'-b',Eval_point,PDF_UB22,'--r'),title('Bounds of E(Y_0 | Z = High)'),xlim([30,80]),ylim([0,0.3]);


% Simulated lower and upper bounds for the unconditional expectation of
% conjectured ASVAB score (Y1, Y0)
Treatment_base1 = 1;
Treatment_base2 = 0;
nsim = 50000;
[LB1_sim,UB1_sim,LB0_sim,UB0_sim,LB_diff_sim,UB_diff_sim] = MARGINAL_BOUND_2MIV(Response,Treatment,MIV1,MIV2,Treatment_base1,Treatment_base2,MTR_flag,nsim);

% Summary statistics of the posterior distribution
Mean_Y1_LB = mean(LB1_sim);
Mean_Y1_UB = mean(UB1_sim);
Mean_Y0_LB = mean(LB0_sim);
Mean_Y0_UB = mean(UB0_sim);
Mean_diff_LB = mean(LB_diff_sim);
Mean_diff_UB = mean(UB_diff_sim);

Std_Y1_LB = std(LB1_sim);
Std_Y1_UB = std(UB1_sim);
Std_Y0_LB = std(LB0_sim);
Std_Y0_UB = std(UB0_sim);
Std_diff_LB = std(LB_diff_sim);
Std_diff_UB = std(UB_diff_sim);

Median_Y1_LB = median(LB1_sim);
Median_Y1_UB = median(UB1_sim);
Median_Y0_LB = median(LB0_sim);
Median_Y0_UB = median(UB0_sim);
Median_diff_LB = median(LB_diff_sim);
Median_diff_UB = median(UB_diff_sim);

PDF_Y1_LB = ksdensity(LB1_sim,Eval_point);
PDF_Y1_UB = ksdensity(UB1_sim,Eval_point);
PDF_Y0_LB = ksdensity(LB0_sim,Eval_point);
PDF_Y0_UB = ksdensity(UB0_sim,Eval_point);
PDF_diff_LB = ksdensity(LB_diff_sim,Eval_point);
PDF_diff_UB = ksdensity(UB_diff_sim,Eval_point);

[temp,ind] = max(PDF_Y1_LB); Mode_Y1_LB = Eval_point(ind);
[temp,ind] = max(PDF_Y1_UB); Mode_Y1_UB = Eval_point(ind);
[temp,ind] = max(PDF_Y0_LB); Mode_Y0_LB = Eval_point(ind);
[temp,ind] = max(PDF_Y0_UB); Mode_Y0_UB = Eval_point(ind);
[temp,ind] = max(PDF_diff_LB); Mode_diff_LB = Eval_point(ind);
[temp,ind] = max(PDF_diff_UB); Mode_diff_UB = Eval_point(ind);

alpha = 0.05;
HPD_Y1_LB = HPD_SIM(LB1_sim, alpha);
HPD_Y1_UB = HPD_SIM(UB1_sim, alpha);
HPD_Y0_LB = HPD_SIM(LB0_sim, alpha);
HPD_Y0_UB = HPD_SIM(UB0_sim, alpha);
HPD_diff_LB = HPD_SIM(LB_diff_sim, alpha);
HPD_diff_UB = HPD_SIM(UB_diff_sim, alpha);

disp(' ')
disp('------------------')
disp('Bounds of Y1 and Y0')
disp('------------------')
fprintf('\t\t\t  Mean \t Std \t Median \t Mode \t\t HPD\n');
disp('Distribution of lower bounds')
fprintf('E( Y1 )  \t %5.2f \t %5.2f \t %5.2f \t %5.2f \t [%5.2f , %5.2f] \n',Mean_Y1_LB,Std_Y1_LB,Median_Y1_LB,Mode_Y1_LB,HPD_Y1_LB(1),HPD_Y1_LB(2));
fprintf('E( Y0 )  \t %5.2f \t %5.2f \t %5.2f \t %5.2f \t [%5.2f , %5.2f] \n',Mean_Y0_LB,Std_Y0_LB,Median_Y0_LB,Mode_Y0_LB,HPD_Y0_LB(1),HPD_Y0_LB(2));
fprintf('E(Y1-Y0) \t %5.2f \t %5.2f \t %5.2f \t %5.2f \t [%5.2f , %5.2f] \n',Mean_diff_LB,Std_diff_LB,Median_diff_LB,Mode_diff_LB,HPD_diff_LB(1),HPD_diff_LB(2));
disp(' ')
disp('Distribution of upper bounds')
fprintf('E( Y1 )  \t %5.2f \t %5.2f \t %5.2f \t %5.2f \t [%5.2f , %5.2f] \n',Mean_Y1_UB,Std_Y1_UB,Median_Y1_UB,Mode_Y1_UB,HPD_Y1_UB(1),HPD_Y1_UB(2));
fprintf('E( Y0 )  \t %5.2f \t %5.2f \t %5.2f \t %5.2f \t [%5.2f , %5.2f] \n',Mean_Y0_UB,Std_Y0_UB,Median_Y0_UB,Mode_Y0_UB,HPD_Y0_UB(1),HPD_Y0_UB(2));
fprintf('E(Y1-Y0) \t %5.2f \t %5.2f \t %5.2f \t %5.2f \t [%5.2f , %5.2f] \n',Mean_diff_UB,Std_diff_UB,Median_diff_UB,Mode_diff_UB,HPD_diff_UB(1),HPD_diff_UB(2));

Marginal_mat = NaN(7,4);
Marginal_mat(1:3,:) = [Mean_Y1_LB,Std_Y1_LB,Median_Y1_LB,Mode_Y1_LB; Mean_Y0_LB,Std_Y0_LB,Median_Y0_LB,Mode_Y0_LB; Mean_diff_LB,Std_diff_LB,Median_diff_LB,Mode_diff_LB];
Marginal_mat(5:7,:) = [Mean_Y1_UB,Std_Y1_UB,Median_Y1_UB,Mode_Y1_UB; Mean_Y0_UB,Std_Y0_UB,Median_Y0_UB,Mode_Y0_UB; Mean_diff_UB,Std_diff_UB,Median_diff_UB,Mode_diff_UB];
HPD_results = cell(7,1);
HPD_results{1} = [ '['  num2str(HPD_Y1_LB(1),'%5.2f')  ' , '  num2str(HPD_Y1_LB(2),'%5.2f')  ']' ];
HPD_results{2} = [ '['  num2str(HPD_Y0_LB(1),'%5.2f')  ' , '  num2str(HPD_Y0_LB(2),'%5.2f')  ']' ];
HPD_results{3} = [ '['  num2str(HPD_diff_LB(1),'%5.2f')  ' , '  num2str(HPD_diff_LB(2),'%5.2f')  ']' ];
HPD_results{5} = [ '['  num2str(HPD_Y1_UB(1),'%5.2f')  ' , '  num2str(HPD_Y1_UB(2),'%5.2f')  ']' ];
HPD_results{6} = [ '['  num2str(HPD_Y0_UB(1),'%5.2f')  ' , '  num2str(HPD_Y0_UB(2),'%5.2f')  ']' ];
HPD_results{7} = [ '['  num2str(HPD_diff_UB(1),'%5.2f')  ' , '  num2str(HPD_diff_UB(2),'%5.2f')  ']' ];
xlswrite('Marginal distribution.xls',Marginal_mat,'Sheet1','B3');
xlswrite('Marginal distribution.xls',HPD_results,'Sheet1','F3');
xlswrite('Marginal distribution.xls',{'Mean','Std','Median','Mode','HPD'},'Sheet1','B1');
xlswrite('Marginal distribution.xls',{'Distribution of lower bounds';'E(Y1)';'E(Y0)';'E(Y1-Y0)';'Distribution of upper bounds';'E(Y1)';'E(Y0)';'E(Y1-Y0)'},'Sheet1','A2');

figure(2)
subplot(1,2,1);
plot(Eval_point,PDF_Y1_LB,'-b',Eval_point,PDF_Y1_UB,'--r'),title('Bounds of E(Y_1)'),xlim([30,80]),ylim([0,0.5]);
legend('Lower bound', 'Upper bound','location','Northwest');xlabel('ASVAB score');
subplot(1,2,2);
plot(Eval_point,PDF_Y0_LB,'-b',Eval_point,PDF_Y0_UB,'--r'),title('Bounds of E(Y_0)'),xlim([30,80]),ylim([0,0.5]);
legend('Lower bound', 'Upper bound','location','Northwest');xlabel('ASVAB score');

figure(3)
plot(Eval_point,PDF_diff_LB,'-b',Eval_point,PDF_diff_UB,'--r'),title('Bounds of E(Y_1) - E(Y_0)'),xlim([0,50]),ylim([0,0.3]);
legend('Lower bound', 'Upper bound','location','Northwest');xlabel('ASVAB score');

